#include <stdio.h>
#include <kdb.h>

int main() {
	KeySet *ks=ksNew();
	Key *key=0;
	KDBHandle handle;
	
	kdbOpen(&handle);
	ksAppend(ks,keyNew("user/sw",KEY_SWITCH_END));  /* a simple key */
	
	ksAppend(ks,keyNew(0));     /* an empty key */
	
	ksAppend(ks,keyNew("system/sw",
		KEY_SWITCH_NEEDSYNC, handle,        /* a key we'll retrieve from storage */
		KEY_SWITCH_END));
	
	ksAppend(ks,keyNew("user/tmp/ex1",
		KEY_SWITCH_VALUE,"some data",        /* with a simple value */
		KEY_SWITCH_END));                    /* end of args */
	
	ksAppend(ks,keyNew("user/tmp/ex2",
		KEY_SWITCH_VALUE,"some data",        /* with a simple value */
		KEY_SWITCH_MODE,0777,                /* permissions */
		KEY_SWITCH_END));                    /* end of args */
	
	ksAppend(ks,keyNew("user/tmp/ex3",
		KEY_SWITCH_TYPE,KEY_TYPE_LINK,       /* only type */
		KEY_SWITCH_VALUE,"system/mtp/x",     /* link destination */
		KEY_SWITCH_MODE,0654,                /* weird permissions */
		KEY_SWITCH_END));                    /* end of args */
	
	ksAppend(ks,keyNew("user/tmp/ex4",
		KEY_SWITCH_TYPE,KEY_TYPE_BINARY,     /* type and value size */
		KEY_SWITCH_COMMENT,"value is truncated",
		KEY_SWITCH_VALUE,"some data",7,      /* value that will be truncated to 7 bytes */
		KEY_SWITCH_UID,0,                    /* root uid */
		KEY_SWITCH_END));                    /* end of args */
	
	ksAppend(ks,keyNew("user/tmp/ex5",
		KEY_SWITCH_VALUE,"some data",        /* value  */
		KEY_SWITCH_DOMAIN,"root",            /* owner (not uid) is root */
		KEY_SWITCH_COMMENT,"some comment",   /* a comment */
		KEY_SWITCH_END));                    /* end of args */
	
	ksAppend(ks,keyNew("user/env/alias/ls",  /* a key we know we have */
		KEY_SWITCH_NEEDSYNC, handle,         /* retrieve from storage */
		KEY_SWITCH_END));                    /* do nothing more */
	
	ksAppend(ks,keyNew("user/env/alias/ls",  /* same key, to compare in output */
		KEY_SWITCH_NEEDSYNC, handle,         /* retrieve from storage */
		KEY_SWITCH_DOMAIN,"root",            /* set new owner (not uid) as root */
		KEY_SWITCH_COMMENT,"new comment",    /* set new comment */
		KEY_SWITCH_END));                    /* end of args */
	
	key=keyNew("user/test//",
		KEY_SWITCH_TYPE, KEY_TYPE_STRING,
		KEY_SWITCH_END);
	
	/* we are providing a lot of '/' to see it being removed */
	keySetName(key,"system");
	keySetName(key,"user");
	keySetName(key,"user:aviram");
	keySetName(key,"user///abc//////def///");
	keySetName(key,"user:root///aaa//////bbb///");
	
	keyAddBaseName(key,"tmp");
	
	keyAddBaseName(key,"////ex6///exx7///");
	
	keySetBaseName(key,"ex8");
	
	keySetBaseName(key,"ex9");
	
	keyAddBaseName(key,"///exxx9///ex10///ex\\/11///");
	
	keySetBaseName(key,"ex12");
	
	keySetBaseName(key,"ex13///");
	
	ksAppend(ks,key);
	
	ksToStream(ks,stdout,KDB_O_XMLHEADERS | KDB_O_FULLNAME);
	
	ksDel(ks);
	
	return kdbClose(&handle);
}



/*
 * Running this program gives me the following output:
 *
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE keyset PUBLIC "-//Avi Alkalay//DTD Elektra 0.1.0//EN" "http://elektra.sf.net/dtd/elektra.dtd">


<!-- Generated by Elektra API. Total of 10 keys. -->



<keyset>


<key name="user:aviram/sw"
     type="0" uid="aviram" gid="aviram" mode="0664">
</key>





<key name=""
     type="0" uid="aviram" gid="aviram" mode="0664">
</key>





<key name="system/sw"
     type="directory" uid="root" gid="root" mode="0755">
</key>





<key name="user:aviram/tmp/ex1"
     type="string" uid="aviram" gid="aviram" mode="0664">

     <value><![CDATA[some data]]></value>
</key>





<key name="user:aviram/tmp/ex2"
     type="string" uid="aviram" gid="aviram" mode="0777">

     <value><![CDATA[some data]]></value>
</key>





<key name="user:aviram/tmp/ex3"
     type="link" uid="aviram" gid="aviram" mode="0654">

     <value><![CDATA[system/mtp/x]]></value>
</key>





<key name="user:aviram/tmp/ex4"
     type="binary" uid="root" gid="aviram" mode="0664">

     <value><![CDATA[some da]]></value>
     <comment><![CDATA[value is truncated]]></comment>
</key>





<key name="user:root/tmp/ex5"
     type="string" uid="aviram" gid="aviram" mode="0664">

     <value><![CDATA[some data]]></value>
     <comment><![CDATA[some comment]]></comment>
</key>





<key name="user:aviram/env/alias/ls"
     type="string" uid="aviram" gid="aviram" mode="0644">

     <value><![CDATA[ls -Fh --color=tty]]></value>
     <comment><![CDATA[The alias for the 'ls' command.
-h to show K and M bytes
-F to show file type indicator
--color=tty to show files in colors

]]></comment>
</key>





<key name="user:root/env/alias/ls"
     type="string" uid="aviram" gid="aviram" mode="0644">

     <value><![CDATA[ls -Fh --color=tty]]></value>
     <comment><![CDATA[new comment]]></comment>
</key>





</keyset>
 *
 */
